"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useMarkdownEditor = void 0;
const react_1 = require("react");
const extensions_1 = require("../extensions");
const logger_1 = require("../logger");
const Editor_1 = require("./Editor");
const wysiwyg_preset_1 = require("./wysiwyg-preset");
// [major] TODO: remove generic type
function useMarkdownEditor(props, deps = []) {
    const editor = (0, react_1.useMemo)(() => {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s;
        const { md = {}, initial = {}, handlers = {}, experimental = {}, markupConfig = {}, wysiwygConfig = {}, } = props;
        const breaks = (_a = md.breaks) !== null && _a !== void 0 ? _a : props.breaks;
        const preset = (_b = props.preset) !== null && _b !== void 0 ? _b : 'full';
        const renderStorage = new extensions_1.ReactRenderStorage();
        const uploadFile = (_c = handlers.uploadFile) !== null && _c !== void 0 ? _c : props.fileUploadHandler;
        const needToSetDimensionsForUploadedImages = (_d = experimental.needToSetDimensionsForUploadedImages) !== null && _d !== void 0 ? _d : props.needToSetDimensionsForUploadedImages;
        const enableNewImageSizeCalculation = experimental.enableNewImageSizeCalculation;
        const extensions = (builder) => {
            var _a;
            const extensionOptions = (_a = wysiwygConfig.extensionOptions) !== null && _a !== void 0 ? _a : props.extensionOptions;
            builder.use(wysiwyg_preset_1.BundlePreset, Object.assign(Object.assign({}, extensionOptions), { preset, reactRenderer: renderStorage, onCancel: () => {
                    editor.emit('cancel', null);
                    return true;
                }, onSubmit: () => {
                    editor.emit('submit', null);
                    return true;
                }, mdBreaks: breaks, fileUploadHandler: uploadFile, needToSetDimensionsForUploadedImages,
                enableNewImageSizeCalculation }));
            {
                const extraExtensions = wysiwygConfig.extensions || props.extraExtensions;
                if (extraExtensions) {
                    builder.use(extraExtensions, props.extensionOptions);
                }
            }
        };
        return new Editor_1.EditorImpl(Object.assign(Object.assign({}, props), { preset,
            renderStorage, md: Object.assign(Object.assign({}, md), { breaks, html: (_e = md.html) !== null && _e !== void 0 ? _e : props.allowHTML, linkify: (_f = md.linkify) !== null && _f !== void 0 ? _f : props.linkify, linkifyTlds: (_g = md.linkifyTlds) !== null && _g !== void 0 ? _g : props.linkifyTlds }), initial: Object.assign(Object.assign({}, initial), { markup: (_h = initial.markup) !== null && _h !== void 0 ? _h : props.initialMarkup, mode: (_j = initial.mode) !== null && _j !== void 0 ? _j : props.initialEditorMode, toolbarVisible: (_k = initial.toolbarVisible) !== null && _k !== void 0 ? _k : props.initialToolbarVisible, splitModeEnabled: (_l = initial.splitModeEnabled) !== null && _l !== void 0 ? _l : props.initialSplitModeEnabled }), handlers: Object.assign(Object.assign({}, handlers), { uploadFile }), experimental: Object.assign(Object.assign({}, experimental), { needToSetDimensionsForUploadedImages,
                enableNewImageSizeCalculation, prepareRawMarkup: (_m = experimental.prepareRawMarkup) !== null && _m !== void 0 ? _m : props.prepareRawMarkup, beforeEditorModeChange: (_o = experimental.beforeEditorModeChange) !== null && _o !== void 0 ? _o : props.experimental_beforeEditorModeChange }), markupConfig: Object.assign(Object.assign({}, markupConfig), { splitMode: (_p = markupConfig.splitMode) !== null && _p !== void 0 ? _p : props.splitMode, renderPreview: (_q = markupConfig.renderPreview) !== null && _q !== void 0 ? _q : props.renderPreview, extensions: (_r = markupConfig.extensions) !== null && _r !== void 0 ? _r : props.extraMarkupExtensions }), wysiwygConfig: Object.assign(Object.assign({}, wysiwygConfig), { extensions, escapeConfig: (_s = wysiwygConfig.escapeConfig) !== null && _s !== void 0 ? _s : props.escapeConfig }) }));
    }, deps);
    (0, react_1.useLayoutEffect)(() => {
        function onToolbarAction({ editorMode, id }) {
            logger_1.logger.action({ mode: editorMode, source: 'toolbar', action: id });
        }
        editor.on('toolbar-action', onToolbarAction);
        return () => {
            editor.off('toolbar-action', onToolbarAction);
            editor.destroy();
        };
    }, [editor]);
    return editor;
}
exports.useMarkdownEditor = useMarkdownEditor;
