"use strict";
var _WysiwygEditor_view, _WysiwygEditor_serializer, _WysiwygEditor_parser, _WysiwygEditor_actions, _WysiwygEditor_contentHandler, _WysiwygEditor_escapeConfig;
Object.defineProperty(exports, "__esModule", { value: true });
exports.WysiwygEditor = void 0;
const tslib_1 = require("tslib");
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_view_1 = require("prosemirror-view");
const ContentHandler_1 = require("./ContentHandler");
const ExtensionsManager_1 = require("./ExtensionsManager");
const actions_1 = require("./utils/actions");
const metrics_1 = require("./utils/metrics");
class WysiwygEditor {
    constructor({ domElem, initialContent = '', extensions = () => { }, allowHTML, mdPreset, linkify, linkifyTlds, escapeConfig, onChange, onDocChange, }) {
        _WysiwygEditor_view.set(this, void 0);
        _WysiwygEditor_serializer.set(this, void 0);
        _WysiwygEditor_parser.set(this, void 0);
        _WysiwygEditor_actions.set(this, void 0);
        _WysiwygEditor_contentHandler.set(this, void 0);
        _WysiwygEditor_escapeConfig.set(this, void 0);
        const { schema, markupParser: parser, serializer, nodeViews, markViews, plugins, rawActions, actions, } = ExtensionsManager_1.ExtensionsManager.process(extensions, {
            // "breaks" option only affects the renderer, but not the parser
            mdOpts: { html: allowHTML, linkify, breaks: true, preset: mdPreset },
            linkifyTlds,
        });
        const state = prosemirror_state_1.EditorState.create({
            schema,
            doc: parser.parse(initialContent),
            plugins,
        });
        const thisOnChange = () => this.tryOnChange(onChange);
        const thisOnDocChange = () => this.tryOnChange(onDocChange);
        tslib_1.__classPrivateFieldSet(this, _WysiwygEditor_view, new prosemirror_view_1.EditorView(domElem !== null && domElem !== void 0 ? domElem : null, {
            state,
            nodeViews,
            markViews,
            dispatchTransaction(tr) {
                const newState = this.state.apply(tr);
                // @ts-expect-error
                this.updateState(newState);
                thisOnChange();
                if (tr.docChanged) {
                    thisOnDocChange();
                }
                (0, metrics_1.logTransactionMetrics)(tr);
            },
        }), "f");
        tslib_1.__classPrivateFieldSet(this, _WysiwygEditor_actions, actions.setActions((0, actions_1.bindActions)(rawActions)(tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_view, "f"))), "f");
        tslib_1.__classPrivateFieldSet(this, _WysiwygEditor_serializer, serializer, "f");
        tslib_1.__classPrivateFieldSet(this, _WysiwygEditor_parser, parser, "f");
        tslib_1.__classPrivateFieldSet(this, _WysiwygEditor_contentHandler, new ContentHandler_1.WysiwygContentHandler(tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_view, "f"), parser), "f");
        tslib_1.__classPrivateFieldSet(this, _WysiwygEditor_escapeConfig, escapeConfig, "f");
    }
    get dom() {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_view, "f").dom;
    }
    get serializer() {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_serializer, "f");
    }
    get parser() {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_parser, "f");
    }
    get actions() {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_actions, "f").actions;
    }
    /** @internal used for prosemirror-dev-tools in demo */
    get view() {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_view, "f");
    }
    action(actionName) {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_actions, "f").action(actionName);
    }
    focus() {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_view, "f").focus();
    }
    hasFocus() {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_view, "f").hasFocus();
    }
    getValue() {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_serializer, "f").serialize(tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_view, "f").state.doc, tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_escapeConfig, "f"));
    }
    isEmpty() {
        var _a;
        const { doc } = tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_view, "f").state;
        return (doc.childCount === 1 &&
            ((_a = doc.firstChild) === null || _a === void 0 ? void 0 : _a.type.name) === 'paragraph' &&
            doc.firstChild.childCount === 0);
    }
    clear() {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_contentHandler, "f").clear();
    }
    replace(newMarkup) {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_contentHandler, "f").replace(newMarkup);
    }
    prepend(markup) {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_contentHandler, "f").prepend(markup);
    }
    append(markup) {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_contentHandler, "f").append(markup);
    }
    moveCursor(position) {
        return tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_contentHandler, "f").moveCursor(position);
    }
    destroy() {
        tslib_1.__classPrivateFieldGet(this, _WysiwygEditor_view, "f").destroy();
    }
    tryOnChange(cb) {
        if (cb) {
            try {
                cb(this);
            }
            catch (err) {
                console.error(err);
            }
        }
    }
}
exports.WysiwygEditor = WysiwygEditor;
_WysiwygEditor_view = new WeakMap(), _WysiwygEditor_serializer = new WeakMap(), _WysiwygEditor_parser = new WeakMap(), _WysiwygEditor_actions = new WeakMap(), _WysiwygEditor_contentHandler = new WeakMap(), _WysiwygEditor_escapeConfig = new WeakMap();
