import type MarkdownIt from 'markdown-it';
import { inputRules } from 'prosemirror-inputrules';
import type { Plugin } from 'prosemirror-state';
import type { ActionSpec } from './types/actions';
import type { Extension, ExtensionDeps, ExtensionMarkSpec, ExtensionNodeSpec, ExtensionSpec, ExtensionWithOptions } from './types/extension';
import type { Keymap } from './types/keymap';
declare type InputRulesConfig = Parameters<typeof inputRules>[0];
declare type ConfigureMdParams = {
    /**
     * Apply this configurtion to text parser
     *
     * @default true
     */
    text?: boolean;
    /**
     * Apply this configurtion to markup parser
     *
     * @default true
     */
    markup?: boolean;
};
declare type ConfigureMdCallback = (md: MarkdownIt) => MarkdownIt;
declare type AddPmNodeCallback = () => ExtensionNodeSpec;
declare type AddPmMarkCallback = () => ExtensionMarkSpec;
declare type AddPmPluginCallback = (deps: ExtensionDeps) => Plugin | Plugin[];
declare type AddPmKeymapCallback = (deps: ExtensionDeps) => Keymap;
declare type AddPmInputRulesCallback = (deps: ExtensionDeps) => InputRulesConfig;
declare type AddActionCallback = (deps: ExtensionDeps) => ActionSpec;
declare enum Priority {
    Highest = 1000000,
    VeryHigh = 100000,
    High = 10000,
    Medium = 1000,
    Low = 100,
    VeryLow = 10,
    Lowest = 0
}
declare type BuilderContext<T extends object> = {
    has(key: keyof T): boolean;
    get<K extends keyof T>(key: K): T[K] | undefined;
    set<K extends keyof T>(key: K, value: T[K]): BuilderContext<T>;
};
declare global {
    namespace WysiwygEditor {
        interface Context {
        }
    }
}
export declare class ExtensionBuilder {
    #private;
    static createContext(): BuilderContext<WysiwygEditor.Context>;
    static readonly Priority: typeof Priority;
    readonly Priority: typeof Priority;
    /** @deprecated use `ExtensionBuilder.Priority` instead */
    static readonly PluginPriority: typeof Priority;
    /** @deprecated use `builder.Priority` instead */
    readonly PluginPriority: typeof Priority;
    readonly context: BuilderContext<WysiwygEditor.Context>;
    constructor(context?: BuilderContext<WysiwygEditor.Context>);
    use(extension: Extension): this;
    use<T>(extension: ExtensionWithOptions<T>, options: T): this;
    configureMd(cb: ConfigureMdCallback, params?: ConfigureMdParams): this;
    addNode(name: string, cb: AddPmNodeCallback): this;
    addMark(name: string, cb: AddPmMarkCallback, priority?: Priority): this;
    addPlugin(cb: AddPmPluginCallback, priority?: Priority): this;
    addKeymap(cb: AddPmKeymapCallback, priority?: Priority): this;
    addInputRules(cb: AddPmInputRulesCallback, priority?: Priority): this;
    addAction(name: string, cb: AddActionCallback): this;
    build(): ExtensionSpec;
}
export {};
