"use strict";
var _ExtensionsManager_schemaRegistry, _ExtensionsManager_parserRegistry, _ExtensionsManager_serializerRegistry, _ExtensionsManager_nodeViewCreators, _ExtensionsManager_markViewCreators, _ExtensionsManager_mdForMarkup, _ExtensionsManager_mdForText, _ExtensionsManager_extensions, _ExtensionsManager_builder, _ExtensionsManager_spec, _ExtensionsManager_deps, _ExtensionsManager_plugins, _ExtensionsManager_actions, _ExtensionsManager_nodeViews, _ExtensionsManager_markViews;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExtensionsManager = void 0;
const tslib_1 = require("tslib");
const markdown_it_1 = tslib_1.__importDefault(require("markdown-it"));
const ActionsManager_1 = require("./ActionsManager");
const ExtensionBuilder_1 = require("./ExtensionBuilder");
const ParserTokensRegistry_1 = require("./ParserTokensRegistry");
const SchemaSpecRegistry_1 = require("./SchemaSpecRegistry");
const SerializerTokensRegistry_1 = require("./SerializerTokensRegistry");
class ExtensionsManager {
    constructor({ extensions, options = {} }) {
        var _a, _b, _c, _d;
        _ExtensionsManager_schemaRegistry.set(this, new SchemaSpecRegistry_1.SchemaSpecRegistry());
        _ExtensionsManager_parserRegistry.set(this, new ParserTokensRegistry_1.ParserTokensRegistry());
        _ExtensionsManager_serializerRegistry.set(this, new SerializerTokensRegistry_1.SerializerTokensRegistry());
        _ExtensionsManager_nodeViewCreators.set(this, new Map());
        _ExtensionsManager_markViewCreators.set(this, new Map());
        _ExtensionsManager_mdForMarkup.set(this, void 0);
        _ExtensionsManager_mdForText.set(this, void 0);
        _ExtensionsManager_extensions.set(this, void 0);
        _ExtensionsManager_builder.set(this, void 0);
        _ExtensionsManager_spec.set(this, void 0);
        _ExtensionsManager_deps.set(this, void 0);
        _ExtensionsManager_plugins.set(this, []);
        _ExtensionsManager_actions.set(this, {});
        _ExtensionsManager_nodeViews.set(this, {});
        _ExtensionsManager_markViews.set(this, {});
        this.processNode = (name, { spec, fromMd, toMd: toMd, view }) => {
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_schemaRegistry, "f").addNode(name, spec);
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_parserRegistry, "f").addToken(fromMd.tokenName || name, fromMd.tokenSpec);
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_serializerRegistry, "f").addNode(name, toMd);
            if (view) {
                tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_nodeViewCreators, "f").set(name, view);
            }
        };
        this.processMark = (name, { spec, fromMd, toMd, view }) => {
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_schemaRegistry, "f").addMark(name, spec);
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_parserRegistry, "f").addToken(fromMd.tokenName || name, fromMd.tokenSpec);
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_serializerRegistry, "f").addMark(name, toMd);
            if (view) {
                tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_markViewCreators, "f").set(name, view);
            }
        };
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_extensions, extensions, "f");
        const mdPreset = (_b = (_a = options.mdOpts) === null || _a === void 0 ? void 0 : _a.preset) !== null && _b !== void 0 ? _b : 'default';
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_mdForMarkup, new markdown_it_1.default(mdPreset, (_c = options.mdOpts) !== null && _c !== void 0 ? _c : {}), "f");
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_mdForText, new markdown_it_1.default(mdPreset, (_d = options.mdOpts) !== null && _d !== void 0 ? _d : {}), "f");
        if (options.linkifyTlds) {
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_mdForMarkup, "f").linkify.tlds(options.linkifyTlds, true);
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_mdForText, "f").linkify.tlds(options.linkifyTlds, true);
        }
        // TODO: add prefilled context
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_builder, new ExtensionBuilder_1.ExtensionBuilder(), "f");
    }
    static process(extensions, options) {
        return new this({ extensions, options }).build();
    }
    build() {
        this.processExtensions();
        this.createDeps();
        this.createDerived();
        return Object.assign(Object.assign({}, tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f")), { actions: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f").actions, rawActions: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_actions, "f"), plugins: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_plugins, "f"), nodeViews: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_nodeViews, "f"), markViews: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_markViews, "f") });
    }
    buildDeps() {
        this.processExtensions();
        this.createDeps();
        return tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f");
    }
    processExtensions() {
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_spec, tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_builder, "f").use(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_extensions, "f")).build(), "f");
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_mdForMarkup, tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_spec, "f").configureMd(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_mdForMarkup, "f"), 'markup'), "f");
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_mdForText, tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_spec, "f").configureMd(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_mdForText, "f"), 'text'), "f");
        tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_spec, "f").nodes().forEach(this.processNode);
        tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_spec, "f").marks().forEach(this.processMark);
    }
    createDeps() {
        const schema = tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_schemaRegistry, "f").createSchema();
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_deps, {
            schema,
            actions: new ActionsManager_1.ActionsManager(),
            markupParser: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_parserRegistry, "f").createParser(schema, tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_mdForMarkup, "f")),
            textParser: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_parserRegistry, "f").createParser(schema, tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_mdForText, "f")),
            serializer: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_serializerRegistry, "f").createSerializer(),
        }, "f");
    }
    createDerived() {
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_plugins, tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_spec, "f").plugins(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f")), "f");
        Object.assign(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_actions, "f"), tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_spec, "f").actions(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f")));
        for (const [name, view] of tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_nodeViewCreators, "f")) {
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_nodeViews, "f")[name] = view(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f"));
        }
        for (const [name, view] of tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_markViewCreators, "f")) {
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_markViews, "f")[name] = view(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f"));
        }
    }
}
exports.ExtensionsManager = ExtensionsManager;
_ExtensionsManager_schemaRegistry = new WeakMap(), _ExtensionsManager_parserRegistry = new WeakMap(), _ExtensionsManager_serializerRegistry = new WeakMap(), _ExtensionsManager_nodeViewCreators = new WeakMap(), _ExtensionsManager_markViewCreators = new WeakMap(), _ExtensionsManager_mdForMarkup = new WeakMap(), _ExtensionsManager_mdForText = new WeakMap(), _ExtensionsManager_extensions = new WeakMap(), _ExtensionsManager_builder = new WeakMap(), _ExtensionsManager_spec = new WeakMap(), _ExtensionsManager_deps = new WeakMap(), _ExtensionsManager_plugins = new WeakMap(), _ExtensionsManager_actions = new WeakMap(), _ExtensionsManager_nodeViews = new WeakMap(), _ExtensionsManager_markViews = new WeakMap();
