"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MermaidView = exports.cnDiagramHelper = exports.cnMermaid = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const icons_1 = require("@gravity-ui/icons");
const uikit_1 = require("@gravity-ui/uikit");
const classname_1 = require("../../../../classname");
const TextInput_1 = require("../../../../forms/TextInput");
const common_1 = require("../../../../i18n/common");
const react_utils_1 = require("../../../../react-utils");
const utils_1 = require("../../../../utils");
const const_1 = require("../MermaidSpecs/const");
exports.cnMermaid = (0, classname_1.cn)('Mermaid');
exports.cnDiagramHelper = (0, classname_1.cn)('MermaidHelper');
const b = exports.cnMermaid;
const MermaidPreview = ({ mermaidInstance, text = '', }) => {
    const [svg, setSvg] = (0, react_1.useState)();
    const [error, setError] = (0, react_1.useState)(null);
    (0, react_1.useEffect)(() => {
        const p = async () => {
            if (mermaidInstance) {
                try {
                    // Validates syntax and throws error if text is invalid
                    await mermaidInstance.parse(text);
                    const { svg: S } = await mermaidInstance.render(`mermaid-${Date.now()}`, text);
                    setSvg(S);
                    setError(null);
                }
                catch (e) {
                    setError(e.message);
                }
            }
        };
        p();
    }, [mermaidInstance, text]);
    if (error) {
        return react_1.default.createElement("div", { className: b('Error') }, error && react_1.default.createElement("div", null, error));
    }
    return (react_1.default.createElement("div", { className: b('Preview') }, svg ? react_1.default.createElement("div", { className: "mermaid", dangerouslySetInnerHTML: { __html: svg } }) : react_1.default.createElement(uikit_1.Loader, null)));
};
const DiagramEditMode = ({ initialText, onSave, onCancel, mermaidInstance }) => {
    const [text, setText] = (0, react_1.useState)(initialText || '');
    return (react_1.default.createElement("div", { className: b() },
        react_1.default.createElement(MermaidPreview, { mermaidInstance: mermaidInstance, text: text }),
        react_1.default.createElement("div", { className: b('Editor') },
            react_1.default.createElement("div", null,
                react_1.default.createElement(TextInput_1.TextAreaFixed, { controlProps: {
                        className: (0, exports.cnDiagramHelper)({ 'prosemirror-stop-event': true }),
                    }, value: text, onUpdate: (v) => {
                        setText(v);
                    }, autoFocus: true })),
            react_1.default.createElement("div", { className: b('Controls') },
                react_1.default.createElement("div", null,
                    react_1.default.createElement(uikit_1.Button, { onClick: onCancel, view: 'flat' },
                        react_1.default.createElement("span", { className: (0, exports.cnDiagramHelper)({ 'prosemirror-stop-event': true }) }, (0, common_1.i18n)('cancel'))),
                    react_1.default.createElement(uikit_1.Button, { onClick: () => onSave(text), view: 'action' },
                        react_1.default.createElement("span", { className: (0, exports.cnDiagramHelper)({ 'prosemirror-stop-event': true }) }, (0, common_1.i18n)('save'))))))));
};
const MermaidView = ({ onChange, node, getPos, view, getMermaidInstance }) => {
    const [mermaidInstance, setMermaidInstance] = (0, react_1.useState)(null);
    const [editing, setEditing, unsetEditing, toggleEditing] = (0, react_utils_1.useBooleanState)(Boolean(node.attrs[const_1.MermaidConsts.NodeAttrs.newCreated]));
    const [menuOpen, , , toggleMenuOpen] = (0, react_utils_1.useBooleanState)(false);
    const buttonRef = (0, react_1.useRef)(null);
    (0, react_1.useEffect)(() => {
        const waitForMermaid = () => setTimeout(() => {
            const instance = getMermaidInstance();
            if (instance) {
                setMermaidInstance(instance);
                return;
            }
            waitForMermaid();
        }, 100);
        waitForMermaid();
    }, []);
    if (editing) {
        return (react_1.default.createElement(DiagramEditMode, { initialText: node.attrs[const_1.MermaidConsts.NodeAttrs.content], mermaidInstance: mermaidInstance, onCancel: unsetEditing, onSave: (v) => {
                onChange({ [const_1.MermaidConsts.NodeAttrs.content]: v });
                unsetEditing();
            } }));
    }
    return (react_1.default.createElement("div", { className: b(), onDoubleClick: setEditing },
        react_1.default.createElement(MermaidPreview, { mermaidInstance: mermaidInstance, text: node.attrs[const_1.MermaidConsts.NodeAttrs.content] }),
        react_1.default.createElement("div", null,
            react_1.default.createElement(uikit_1.Button, { onClick: toggleMenuOpen, ref: buttonRef, size: 's', className: (0, exports.cnDiagramHelper)({ 'prosemirror-stop-event': true }) },
                react_1.default.createElement(uikit_1.Icon, { data: icons_1.Ellipsis, className: (0, exports.cnDiagramHelper)({ 'prosemirror-stop-event': true }) })),
            react_1.default.createElement(uikit_1.Popup, { anchorRef: buttonRef, open: menuOpen, onClose: toggleMenuOpen, placement: ['bottom-end'] },
                react_1.default.createElement(uikit_1.Menu, null,
                    react_1.default.createElement(uikit_1.Menu.Item, { onClick: () => {
                            toggleEditing();
                            toggleMenuOpen();
                        } }, (0, common_1.i18n)('edit')),
                    react_1.default.createElement(uikit_1.Menu.Item, { onClick: () => {
                            const pos = getPos();
                            if (pos === undefined)
                                return;
                            (0, utils_1.removeNode)({
                                node,
                                pos,
                                tr: view.state.tr,
                                dispatch: view.dispatch,
                            });
                        } }, (0, common_1.i18n)('remove')))))));
};
exports.MermaidView = MermaidView;
