"use strict";
var _ReactRenderStorage_items, _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactRenderStorage = exports.ReactRendererExtension = exports.getReactRendererFromState = exports.ReactRendererComponent = void 0;
const tslib_1 = require("tslib");
const prosemirror_state_1 = require("prosemirror-state");
const event_emitter_1 = require("../../../utils/event-emitter");
var react_1 = require("./react");
Object.defineProperty(exports, "ReactRendererComponent", { enumerable: true, get: function () { return react_1.Renderer; } });
const key = new prosemirror_state_1.PluginKey('reactrenderer');
function getReactRendererFromState(state) {
    const renderer = key.getState(state);
    if (!renderer) {
        throw new Error('ReactRenderer is missing in the editor state. Probably, you forgot to use ReactRendererExtension');
    }
    return renderer;
}
exports.getReactRendererFromState = getReactRendererFromState;
const ReactRendererExtension = (builder, renderer) => {
    builder.context.set('reactrenderer', renderer);
    builder.addPlugin(() => new prosemirror_state_1.Plugin({
        key,
        state: {
            init: () => renderer,
            apply: () => renderer,
        },
    }), builder.Priority.Highest);
};
exports.ReactRendererExtension = ReactRendererExtension;
class ReactRenderStorage extends event_emitter_1.EventEmitter {
    constructor() {
        super(...arguments);
        _ReactRenderStorage_items.set(this, []);
    }
    getItems() {
        return tslib_1.__classPrivateFieldGet(this, _ReactRenderStorage_items, "f");
    }
    createItem(idPrefix, render) {
        const item = new ReactRenderStorage.Item(idPrefix, render, this.removeItem.bind(this));
        tslib_1.__classPrivateFieldGet(this, _ReactRenderStorage_items, "f").push(item);
        this.emit('update', null);
        return item;
    }
    removeItem(itemOrId) {
        if (typeof itemOrId === 'string') {
            tslib_1.__classPrivateFieldSet(this, _ReactRenderStorage_items, tslib_1.__classPrivateFieldGet(this, _ReactRenderStorage_items, "f").filter((item) => item.id !== itemOrId), "f");
        }
        else {
            tslib_1.__classPrivateFieldSet(this, _ReactRenderStorage_items, tslib_1.__classPrivateFieldGet(this, _ReactRenderStorage_items, "f").filter((item) => item !== itemOrId), "f");
        }
        this.emit('update', null);
    }
}
exports.ReactRenderStorage = ReactRenderStorage;
_ReactRenderStorage_items = new WeakMap();
ReactRenderStorage.Item = (_a = class Item extends event_emitter_1.EventEmitter {
        constructor(idPrefix, renderFn, removeFn) {
            super();
            this.id = idPrefix + Item.getNextId();
            this.renderFn = renderFn;
            this.removeFn = removeFn;
        }
        static getNextId() {
            return this.GlobalId++;
        }
        render() {
            return this.renderFn();
        }
        remove() {
            this.removeFn(this);
        }
        rerender() {
            this.emit('rerender', null);
        }
    },
    _a.GlobalId = 0,
    _a);
