"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.joinBackwardToOpenTab = exports.removeTabWhenCursorAtTheStartOfTab = exports.removeTab = exports.createTab = exports.liftEmptyBlockFromTabPanel = exports.tabEnter = exports.tabPanelArrowDown = exports.dragAutoSwitch = void 0;
const utils_1 = require("@diplodoc/transform/lib/plugins/utils");
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_utils_1 = require("prosemirror-utils");
const __1 = require("../../");
const lodash_1 = require("../../../lodash");
const helpers_1 = require("../../../table-utils/helpers");
const utils_2 = require("../../../utils");
const selection_1 = require("../../../utils/selection");
const const_1 = require("./YfmTabsSpecs/const");
const const_2 = require("./const");
const utils_3 = require("./utils");
const dragAutoSwitch = () => new prosemirror_state_1.Plugin({
    view: TabsAutoSwitchOnDragOver.view,
});
exports.dragAutoSwitch = dragAutoSwitch;
class TabsAutoSwitchOnDragOver {
    constructor(view) {
        this._tabElem = null;
        this._timeout = null;
        this._editorView = view;
        this._docListener = (0, lodash_1.throttle)(this._onDocEvent.bind(this), TabsAutoSwitchOnDragOver.THROTTLE_WAIT);
        document.addEventListener('mousemove', this._docListener);
        document.addEventListener('dragover', this._docListener);
    }
    destroy() {
        this._clear();
        this._docListener.cancel();
        document.removeEventListener('mousemove', this._docListener);
        document.removeEventListener('dragover', this._docListener);
    }
    _onDocEvent(event) {
        const view = this._editorView;
        if (!view.dragging)
            return;
        const pos = view.posAtCoords({ left: event.clientX, top: event.clientY });
        if (pos) {
            const elem = (0, prosemirror_utils_1.findDomRefAtPos)(pos.pos, view.domAtPos.bind(view));
            const cutElem = elem.closest(TabsAutoSwitchOnDragOver.TAB_SELECTOR);
            if (cutElem === this._tabElem)
                return;
            this._clear();
            if (cutElem)
                this._setTabElem(cutElem);
        }
    }
    _clear() {
        if (this._timeout !== null)
            clearTimeout(this._timeout);
        this._timeout = null;
        this._tabElem = null;
    }
    _setTabElem(elem) {
        this._tabElem = elem;
        this._timeout = setTimeout(this._switchTab.bind(this), TabsAutoSwitchOnDragOver.OPEN_TIMEOUT);
    }
    _switchTab() {
        if (this._editorView.dragging && this._tabElem) {
            const pos = this._editorView.posAtDOM(this._tabElem, 0, -1);
            const $pos = this._editorView.state.doc.resolve(pos);
            const { state } = this._editorView;
            let { depth } = $pos;
            let tabId = '';
            let tabsNode = null;
            do {
                const node = $pos.node(depth);
                if (node.type === (0, const_2.tabType)(state.schema)) {
                    tabId = node.attrs[const_1.TabAttrs.dataDiplodocid];
                    continue;
                }
                if (node.type === (0, const_2.tabsType)(state.schema)) {
                    tabsNode = { node, pos: $pos.before(depth) };
                    break;
                }
            } while (--depth >= 0);
            if (tabId && tabsNode) {
                const { tr } = state;
                if (switchYfmTab(tabsNode, tabId, tr)) {
                    this._editorView.dispatch(tr.setMeta('addToHistory', false));
                }
            }
        }
        this._clear();
    }
}
_a = TabsAutoSwitchOnDragOver;
TabsAutoSwitchOnDragOver.TAB_SELECTOR = '.yfm-tab:not([data-diplodoc-is-active=true])';
TabsAutoSwitchOnDragOver.OPEN_TIMEOUT = 500; //ms
TabsAutoSwitchOnDragOver.THROTTLE_WAIT = 50; //ms
TabsAutoSwitchOnDragOver.view = (view) => new _a(view);
function switchYfmTab({ node: tabsNode, pos: tabsPos }, tabId, tr) {
    const { schema } = tabsNode.type;
    if (tabsNode.type !== (0, const_2.tabsType)(schema))
        return false;
    const tabsList = tabsNode.firstChild;
    if ((tabsList === null || tabsList === void 0 ? void 0 : tabsList.type) !== (0, const_2.tabsListType)(schema))
        return false;
    const tabsListPos = tabsPos + 1;
    let panelId = null;
    tabsList.forEach((node, offset) => {
        if (node.type !== (0, const_2.tabType)(schema))
            return;
        const tabPos = tabsListPos + 1 + offset;
        const tabAttrs = Object.assign(Object.assign({}, node.attrs), { [const_1.TabAttrs.ariaSelected]: 'false', [const_1.TabAttrs.dataDiplodocIsActive]: 'false' });
        if (node.attrs[const_1.TabAttrs.dataDiplodocid] === tabId) {
            panelId = node.attrs[const_1.TabAttrs.ariaControls];
            tabAttrs[const_1.TabAttrs.ariaSelected] = 'true';
            tabAttrs[const_1.TabAttrs.dataDiplodocIsActive] = 'true';
        }
        tr.setNodeMarkup(tabPos, null, tabAttrs);
    });
    if (!panelId)
        return false;
    tabsNode.forEach((node, offset) => {
        var _b;
        if (node.type !== (0, const_2.tabPanelType)(schema))
            return;
        const tabPanelPos = tabsPos + 1 + offset;
        const tabPanelAttrs = Object.assign({}, node.attrs);
        const tabPanelClassList = new Set(((_b = node.attrs[const_1.TabPanelAttrs.class]) !== null && _b !== void 0 ? _b : '')
            .split(' ')
            .filter((val) => Boolean(val.trim())));
        if (node.attrs[const_1.TabPanelAttrs.id] === panelId) {
            tabPanelClassList.add('active');
        }
        else {
            tabPanelClassList.delete('active');
        }
        tabPanelAttrs[const_1.TabPanelAttrs.class] = Array.from(tabPanelClassList).join(' ');
        tr.setNodeMarkup(tabPanelPos, null, tabPanelAttrs);
    });
    return true;
}
const tabPanelArrowDown = (state, dispatch, view) => {
    var _b;
    const { selection: sel } = state;
    const tabsParentNode = (0, prosemirror_utils_1.findParentNodeOfType)((0, const_2.tabsType)(state.schema))(state.selection);
    if ((0, utils_3.atEndOfPanel)(view) && (view === null || view === void 0 ? void 0 : view.endOfTextblock('down'))) {
        const [direction, $cursor] = ['after', sel.$to];
        let $pos;
        if ((0, selection_1.isTextSelection)(sel)) {
            $pos =
                (_b = (0, __1.findFakeParaPosForTextSelection)(sel, direction)) !== null && _b !== void 0 ? _b : (0, __1.findFakeParaPosClosestToPos)($cursor, $cursor.depth - 2, direction);
        }
        if ((0, __1.isGapCursorSelection)(sel) && tabsParentNode) {
            $pos = state.doc.resolve(tabsParentNode.pos + tabsParentNode.node.nodeSize);
        }
        if ($pos) {
            dispatch === null || dispatch === void 0 ? void 0 : dispatch((0, __1.createFakeParagraph)(state.tr, $pos, direction).scrollIntoView());
            return true;
        }
    }
    return false;
};
exports.tabPanelArrowDown = tabPanelArrowDown;
const tabEnter = (state) => {
    const $cursor = (0, selection_1.get$Cursor)(state.selection);
    return ($cursor === null || $cursor === void 0 ? void 0 : $cursor.node($cursor.depth).type) === (0, const_2.tabType)(state.schema);
};
exports.tabEnter = tabEnter;
const liftEmptyBlockFromTabPanel = (state, dispatch) => {
    const { selection, schema } = state;
    if (!(0, selection_1.isTextSelection)(selection))
        return false;
    const { $cursor } = selection;
    // cursor should be inside an empty textblock
    if (!$cursor || $cursor.parent.content.size)
        return false;
    if ($cursor.depth > 2 && // depth must be at least 3
        (0, utils_2.isSameNodeType)($cursor.node(-1), (0, const_2.tabPanelType)(schema)) &&
        (0, utils_2.isSameNodeType)($cursor.node(-2), (0, const_2.tabsType)(schema))) {
        // current texblock is last child
        if ($cursor.after() === $cursor.end(-1)) {
            if (dispatch) {
                const copeidNode = $cursor.parent.copy();
                const tabsAfter = $cursor.after(-2);
                const { tr } = state;
                tr.insert(tabsAfter, copeidNode)
                    .delete($cursor.before(), $cursor.after())
                    .setSelection(prosemirror_state_1.TextSelection.create(tr.doc, tr.mapping.map(tabsAfter)));
                dispatch(tr.scrollIntoView());
            }
            return true;
        }
    }
    return false;
};
exports.liftEmptyBlockFromTabPanel = liftEmptyBlockFromTabPanel;
const makeTabsInactive = (tabNodes, tabPanels, tr) => {
    // Find all active tabs and make them inactive
    const activeTabs = tabNodes.filter((v) => v.node.attrs[const_1.TabAttrs.dataDiplodocIsActive] === 'true');
    if (activeTabs.length) {
        activeTabs.forEach((tab) => {
            tr.setNodeMarkup(tab.pos, null, Object.assign(Object.assign({}, tab.node.attrs), { class: const_2.tabInactiveClassname, [const_1.TabAttrs.dataDiplodocIsActive]: 'false' }));
        });
    }
    // Find all active panels and make them inactive
    const activePanels = tabPanels.filter((v) => v.node.attrs[const_1.TabPanelAttrs.class] === const_2.tabPanelActiveClassname);
    if (activePanels.length) {
        activePanels.forEach((tabPanel) => {
            tr.setNodeMarkup(tr.mapping.map(tabPanel.pos), null, Object.assign(Object.assign({}, tabPanel.node.attrs), { class: const_2.tabPanelInactiveClassname }));
        });
    }
};
const createTab = (afterTab, tabsParentNode) => (state, dispatch, view) => {
    const tabNodes = (0, prosemirror_utils_1.findChildren)(tabsParentNode.node, (node) => node.type.name === (0, const_2.tabType)(state.schema).name);
    const tabPanels = (0, prosemirror_utils_1.findChildren)(tabsParentNode.node, (tabNode) => {
        return tabNode.type.name === (0, const_2.tabPanelType)(state.schema).name;
    });
    const afterPanelNode = tabPanels.filter((tabPanelNode) => tabPanelNode.node.attrs[const_1.TabPanelAttrs.ariaLabelledby] ===
        afterTab.node.attrs[const_1.TabAttrs.dataDiplodocid])[0];
    const tabId = (0, utils_1.generateID)();
    const panelId = (0, utils_1.generateID)();
    const newPanel = (0, const_2.tabPanelType)(state.schema).create({
        [const_1.TabPanelAttrs.ariaLabelledby]: tabId,
        [const_1.TabPanelAttrs.id]: panelId,
        [const_1.TabPanelAttrs.class]: const_2.tabPanelActiveClassname,
    }, (0, __1.pType)(state.schema).createAndFill());
    const newTab = (0, const_2.tabType)(state.schema).create({
        [const_1.TabAttrs.id]: tabId,
        [const_1.TabAttrs.dataDiplodocid]: tabId,
        [const_1.TabAttrs.dataDiplodocKey]: tabId,
        [const_1.TabAttrs.dataDiplodocIsActive]: 'true',
        [const_1.TabAttrs.class]: const_2.tabActiveClassname,
        [const_1.TabAttrs.role]: 'tab',
        [const_1.TabAttrs.ariaControls]: panelId,
    });
    const { tr } = state;
    // Change relative pos to absolute
    tabNodes.forEach((v) => {
        v.pos = v.pos + tabsParentNode.pos + 1;
    });
    tabPanels.forEach((v) => {
        v.pos = v.pos + tabsParentNode.pos + 1;
    });
    makeTabsInactive(tabNodes, tabPanels, tr);
    dispatch === null || dispatch === void 0 ? void 0 : dispatch(tr
        .insert(afterPanelNode.pos + afterPanelNode.node.nodeSize, newPanel)
        .insert(afterTab.pos + afterTab.node.nodeSize, newTab)
        .setSelection(prosemirror_state_1.TextSelection.create(tr.doc, afterTab.pos + afterTab.node.nodeSize + 1)));
    view === null || view === void 0 ? void 0 : view.focus();
    return true;
};
exports.createTab = createTab;
const removeTab = (tabToRemove, tabsParentNode) => (state, dispatch, view) => {
    const tabList = (0, prosemirror_utils_1.findChildren)(tabsParentNode.node, (tabNode) => {
        return tabNode.type.name === (0, const_2.tabsListType)(state.schema).name;
    })[0];
    const tabToRemoveIdx = (0, helpers_1.findChildIndex)(tabList.node, tabToRemove.node);
    const tabNodes = (0, prosemirror_utils_1.findChildren)(tabList.node, (node) => node.type.name === (0, const_2.tabType)(state.schema).name);
    const tabPanels = (0, prosemirror_utils_1.findChildren)(tabsParentNode.node, (tabNode) => {
        return tabNode.type.name === (0, const_2.tabPanelType)(state.schema).name;
    });
    const panelToRemove = tabPanels.filter((tabPanelNode) => tabPanelNode.node.attrs[const_1.TabPanelAttrs.ariaLabelledby] ===
        tabToRemove.node.attrs[const_1.TabAttrs.dataDiplodocid])[0];
    if (panelToRemove && dispatch) {
        const { tr } = state;
        if (tabNodes.length <= 1) {
            tr.delete(tabsParentNode.pos, tabsParentNode.pos + tabsParentNode.node.nodeSize);
        }
        else {
            const newTabIdx = tabToRemoveIdx - 1 < 0 ? 1 : tabToRemoveIdx - 1;
            // Change relative pos to absolute
            tabNodes.forEach((v) => {
                v.pos = v.pos + tabsParentNode.pos + 2;
            });
            tabPanels.forEach((v) => {
                v.pos = v.pos + tabsParentNode.pos + 1;
            });
            const newTabNode = tabNodes[newTabIdx];
            const newTabPanelNode = tabPanels[newTabIdx];
            makeTabsInactive(tabNodes, tabPanels, tr);
            tr
                // Delete panel
                .delete(panelToRemove.pos, panelToRemove.pos + panelToRemove.node.nodeSize)
                // Delete tab
                .delete(tabToRemove.pos, tabToRemove.pos + tabToRemove.node.nodeSize)
                // Set new active tab
                .setNodeMarkup(tr.mapping.map(newTabNode.pos), null, Object.assign(Object.assign({}, newTabNode.node.attrs), { class: const_2.tabActiveClassname, [const_1.TabAttrs.dataDiplodocIsActive]: 'true' }))
                // Set new active panel
                .setNodeMarkup(tr.mapping.map(newTabPanelNode.pos), null, Object.assign(Object.assign({}, newTabPanelNode.node.attrs), { class: const_2.tabPanelActiveClassname }))
                .setSelection(prosemirror_state_1.TextSelection.create(tr.doc, tr.mapping.map(newTabNode.pos + newTabNode.node.nodeSize - 1)));
        }
        dispatch(tr);
        view === null || view === void 0 ? void 0 : view.focus();
        return true;
    }
    return false;
};
exports.removeTab = removeTab;
const removeTabWhenCursorAtTheStartOfTab = (state, dispatch) => {
    const tabToRemove = (0, prosemirror_utils_1.findParentNodeOfType)((0, const_2.tabType)(state.schema))(state.selection);
    const tabsParentNode = (0, prosemirror_utils_1.findParentNodeOfType)((0, const_2.tabsType)(state.schema))(state.selection);
    if (tabsParentNode &&
        tabToRemove &&
        state.selection.from === tabToRemove.pos + 1 &&
        state.selection.from === state.selection.to) {
        return (0, exports.removeTab)(tabToRemove, tabsParentNode)(state, dispatch);
    }
    return false;
};
exports.removeTabWhenCursorAtTheStartOfTab = removeTabWhenCursorAtTheStartOfTab;
const joinBackwardToOpenTab = (state, dispatch) => {
    const $cursor = (0, selection_1.get$Cursor)(state.selection);
    if (!$cursor || $cursor.parentOffset !== 0)
        return false;
    // --> cursor at start of textblock
    const textBlockIndex = $cursor.index($cursor.depth - 1);
    if (textBlockIndex <= 0)
        return false;
    const nodeBefore = $cursor.node($cursor.depth - 1).child(textBlockIndex - 1);
    if (nodeBefore.type !== (0, const_2.tabsType)(state.schema)) {
        return false;
    }
    const tabsParent = (0, prosemirror_utils_1.findParentNodeOfTypeClosestToPos)(state.doc.resolve($cursor.pos - 2), (0, const_2.tabsType)(state.schema));
    if (!tabsParent) {
        return false;
    }
    const activePanel = (0, prosemirror_utils_1.findChildren)(tabsParent.node, (n) => n.attrs.class === const_2.tabPanelActiveClassname)[0];
    if (dispatch) {
        const posEndOfLastLayoutCell = activePanel.pos + tabsParent.pos + activePanel.node.nodeSize;
        const tr = state.tr;
        tr.delete($cursor.before(), $cursor.after());
        tr.insert(posEndOfLastLayoutCell, $cursor.parent);
        tr.setSelection(prosemirror_state_1.TextSelection.create(tr.doc, posEndOfLastLayoutCell + 1));
        dispatch(tr.scrollIntoView());
    }
    return true;
};
exports.joinBackwardToOpenTab = joinBackwardToOpenTab;
