/* eslint-disable jsx-a11y/no-static-element-interactions */
/* eslint-disable jsx-a11y/click-events-have-key-events */
import React from 'react';
import { HelpPopover } from '@gravity-ui/components';
import { Eye, Gear, LogoMarkdown } from '@gravity-ui/icons';
import { ActionTooltip, Button, Checkbox, Icon, Menu, Popup, } from '@gravity-ui/uikit';
import { cn } from '../../classname';
import { i18n } from '../../i18n/bundle';
import WysiwygModeIcon from '../../icons/WysiwygMode';
import { noop } from '../../lodash';
import { useBooleanState } from '../../react-utils/hooks';
import { ToolbarTooltipDelay } from '../../toolbar';
import { VERSION } from '../../version';
import { MarkdownHints } from './MarkdownHints';
import './index.css';
const placement = ['bottom-end', 'top-end'];
const bSettings = cn('editor-settings');
const bContent = cn('settings-content');
export const EditorSettings = React.memo(function EditorSettings(props) {
    const { className, onShowPreviewChange, showPreview, renderPreviewButton, settingsVisible } = props;
    const chevronRef = React.useRef(null);
    const [popupShown, , hidePopup, togglePopup] = useBooleanState(false);
    return (React.createElement("div", { className: bSettings(null, [className]) },
        renderPreviewButton && (React.createElement(React.Fragment, null,
            React.createElement(ActionTooltip, { openDelay: ToolbarTooltipDelay.Open, closeDelay: ToolbarTooltipDelay.Close, title: i18n('preview_hint'), hotkey: "mod+shift+p" },
                React.createElement(Button, { size: "m", view: "flat", pin: "round-round", className: bSettings('preview-button'), onClick: () => onShowPreviewChange === null || onShowPreviewChange === void 0 ? void 0 : onShowPreviewChange(!showPreview), selected: showPreview },
                    React.createElement(Icon, { data: Eye }))),
            settingsVisible && React.createElement("div", { className: bSettings('separator') }))),
        settingsVisible && (React.createElement(React.Fragment, null,
            React.createElement(Button, { size: "m", view: "flat", ref: chevronRef, pin: "round-round", onClick: togglePopup, className: bSettings('dropdown-button') },
                React.createElement(Icon, { data: Gear })),
            React.createElement(Popup, { open: popupShown, anchorRef: chevronRef, placement: placement, onClose: hidePopup },
                React.createElement(SettingsContent, Object.assign({}, props, { onClose: hidePopup, className: bSettings('content') })))))));
});
const mdHelpPlacement = ['bottom', 'bottom-end', 'right-start', 'right', 'left'];
const SettingsContent = function SettingsContent({ mode, onClose, onModeChange, toolbarVisibility, onToolbarVisibilityChange, onSplitModeChange, splitMode = false, splitModeEnabled, className, showPreview, }) {
    return (React.createElement("div", { className: bContent(null, [className]) },
        React.createElement(Menu, { size: "l", className: bContent('mode') },
            React.createElement(Menu.Item, { active: mode === 'wysiwyg', onClick: () => {
                    onModeChange('wysiwyg');
                    onClose();
                }, icon: React.createElement(Icon, { data: WysiwygModeIcon }) }, i18n('settings_wysiwyg')),
            React.createElement(Menu.Item, { active: mode === 'markup', onClick: () => {
                    onModeChange('markup');
                    onClose();
                }, icon: React.createElement(Icon, { data: LogoMarkdown }) },
                i18n('settings_markup'),
                React.createElement(HelpPopover, { content: React.createElement("div", { onClick: (e) => {
                            // stop clicks propagation
                            // because otherwise click in MarkdownHints handled as click on MenuItem
                            e.stopPropagation();
                        } },
                        React.createElement(MarkdownHints, null)), placement: mdHelpPlacement, className: bContent('mode-help') }))),
        React.createElement("div", { className: bContent('separator') }),
        !showPreview && (React.createElement("div", { className: bContent('toolbar') },
            React.createElement(Checkbox, { size: "m", checked: toolbarVisibility, onUpdate: onToolbarVisibilityChange }, i18n('settings_menubar')),
            React.createElement("div", { className: bContent('toolbar-hint') }, i18n('settings_hint')))),
        splitMode && (React.createElement("div", { className: bContent('split-mode') },
            React.createElement(Checkbox, { size: "m", disabled: mode !== 'markup', checked: splitModeEnabled, onUpdate: onSplitModeChange !== null && onSplitModeChange !== void 0 ? onSplitModeChange : noop }, i18n('settings_split-mode')),
            React.createElement("div", { className: bContent('toolbar-hint') }, i18n('settings_split-mode-hint')))),
        React.createElement("span", { className: bContent('version') }, VERSION)));
};
