import React, { useCallback } from 'react';
import { Popup } from '@gravity-ui/uikit';
import { ImageForm } from '../../../forms/ImageForm';
import { i18n } from '../../../i18n/forms';
import { useBooleanState } from '../../../react-utils/hooks';
import { useToaster } from '../../../react-utils/toaster';
import { batchUploadFiles } from '../../../utils/upload';
const placement = ['bottom-start', 'top-start', 'bottom-end', 'top-end'];
export const ToolbarImagePopup = ({ className, anchorRef, hide, onSubmit, focus, onClick, uploadImages, onSuccessUpload, }) => {
    const toaster = useToaster();
    const [loading, showLoading, hideLoading] = useBooleanState(false);
    const handleCancel = useCallback(() => {
        hide();
        focus();
    }, [focus, hide]);
    return (React.createElement(Popup, { open: true, anchorRef: anchorRef, onClose: handleCancel, placement: placement, className: className },
        React.createElement(ImageForm, { autoFocus: true, onAttach: uploadImages &&
                ((files) => {
                    showLoading();
                    batchUploadFiles(files, uploadImages).then((res) => {
                        hideLoading();
                        hide();
                        onSuccessUpload === null || onSuccessUpload === void 0 ? void 0 : onSuccessUpload(res);
                    }, (err) => {
                        hideLoading();
                        toaster.add({
                            theme: 'danger',
                            name: 'toolbar_image_upload',
                            title: i18n('image_upload_failed'),
                            content: String(err),
                        });
                    });
                }), onCancel: handleCancel, onSubmit: (data) => {
                hide();
                focus();
                onSubmit(data);
                onClick === null || onClick === void 0 ? void 0 : onClick('addImage');
            }, loading: loading })));
};
