import React, { useCallback, useRef, useState } from 'react';
import { ArrowRight, ArrowRotateLeft, ThumbsDown, ThumbsUp } from '@gravity-ui/icons';
import { ActionTooltip, Alert, Button, Icon, TextInput } from '@gravity-ui/uikit';
import { cn } from '../../../../classname';
import { i18n } from '../../../../i18n/gpt/dialog';
import gptIcon from '../../../../icons/GPT';
import { useAutoFocus } from '../../../../react-utils/useAutoFocus';
import { ErrorScreen } from '../ErrorScreen/ErrorScreen';
import { IconRefuge } from '../IconRefuge/IconRefuge';
import { PresetList } from '../PresetList/PresetList';
import { gptHotKeys } from '../constants';
import { useGpt } from '../hooks/useGpt';
import { useGptHotKeys } from '../hooks/useGptHotKeys';
import { getAlertGptInfo, getDisableReplaceButtonText, getInputPlaceHolder } from '../utils';
import { LoadingScreen } from './LoadingScreen/LoadingScreen';
import './GptDialog.css';
export const cnGptDialog = cn('gpt-dialog');
export const GptDialog = ({ markup, answerRender, promptPresets, disablePromptPresets, customPromptPlaceholder, disabledPromptPlaceholder, onCustomPromptApply, onPromptPresetClick, onTryAgain, onApplyResult, onClose, onLike, onDislike, onUpdate, gptAlertProps, }) => {
    const { answer, customPrompt, loading, mode, feedbackType, feedbackTypeLoading, handleLike, handleDislike, handleCustomPromptUpdate, handleCustomPromptKeyPress, handleCustomPromptApply, handlePresetClick, handleTryAgain, handleFreshStart, handleApplyResult, showAnswer, showError, showAnswerActions, showTryAgainButton, } = useGpt({
        markup,
        promptPresets,
        onLike,
        onDislike,
        onCustomPromptApply,
        onPromptPresetClick,
        onTryAgain,
        onApplyResult,
        onUpdate,
    });
    const gptAlert = gptAlertProps;
    const customPromptContainerRef = useRef(null);
    const [showedGptAlert, setShowedGptAlert] = useState(gptAlert === null || gptAlert === void 0 ? void 0 : gptAlert.showedGptAlert);
    const { alertMessage, alertTheme, alertClassName } = getAlertGptInfo(gptAlert);
    const onCloseAlert = useCallback(() => {
        var _a;
        if (gptAlert) {
            gptAlert.showedGptAlert = false;
        }
        (_a = gptAlert === null || gptAlert === void 0 ? void 0 : gptAlert.onCloseGptAlert) === null || _a === void 0 ? void 0 : _a.call(gptAlert);
        setShowedGptAlert(false);
    }, [gptAlert]);
    useAutoFocus(customPromptContainerRef, [showAnswer]);
    useGptHotKeys(gptHotKeys.tryAgainGpt, handleTryAgain);
    useGptHotKeys(gptHotKeys.freshStartGpt, handleFreshStart);
    useGptHotKeys(gptHotKeys.applyResultGpt, handleApplyResult);
    const replaceButtonText = getDisableReplaceButtonText(disablePromptPresets);
    const inputPlaceholderText = getInputPlaceHolder(disablePromptPresets, disabledPromptPlaceholder, customPromptPlaceholder);
    const tryAgainButton = (React.createElement(ActionTooltip, { hotkey: gptHotKeys.tryAgainGpt, title: i18n('try-again') },
        React.createElement(Button, { className: cnGptDialog('try-again-button'), view: "normal", size: "m", onClick: handleTryAgain },
            React.createElement(Icon, { data: ArrowRotateLeft, size: 16 }),
            i18n('try-again'))));
    let content = null;
    if (loading) {
        content = React.createElement(LoadingScreen, null);
    }
    else if (showError) {
        content = React.createElement(ErrorScreen, { onRetry: handleTryAgain, onStartAgain: handleFreshStart });
    }
    else {
        content = (React.createElement(React.Fragment, null,
            React.createElement("div", { className: cnGptDialog('header') },
                React.createElement("div", { className: cnGptDialog('header-top') },
                    React.createElement(IconRefuge, { containerClassName: cnGptDialog('gpt-icon'), data: gptIcon, refugeSize: 28, size: 16 }),
                    (mode === 'only-custom' || mode === 'custom-and-presets') &&
                        (showAnswer ? (React.createElement("span", { className: cnGptDialog('answer-title') }, i18n('answer-title'))) : (React.createElement("div", { className: cnGptDialog('custom-prompt') },
                            React.createElement(TextInput, { controlRef: customPromptContainerRef, view: "clear", size: "m", className: cnGptDialog('custom-prompt-input'), placeholder: inputPlaceholderText, onKeyPress: handleCustomPromptKeyPress, onUpdate: handleCustomPromptUpdate }),
                            React.createElement(Button, { className: cnGptDialog('custom-prompt-ask-button'), view: "normal", size: "s", disabled: !customPrompt, onClick: handleCustomPromptApply },
                                React.createElement(Icon, { data: ArrowRight, size: 16 }))))),
                    mode === 'only-presets' && (React.createElement("div", { className: cnGptDialog('alone-presets') },
                        React.createElement("span", { className: cnGptDialog('alone-presets-text') }, i18n('only-presets-title'))))),
                (mode === 'custom-and-presets' || mode === 'only-presets') &&
                    ((showTryAgainButton && (React.createElement("div", { className: cnGptDialog('header-bottom') },
                        tryAgainButton,
                        React.createElement(ActionTooltip, { title: i18n('fresh-start-button'), hotkey: gptHotKeys.freshStartGpt },
                            React.createElement(Button, { className: cnGptDialog('back-to-start'), view: "normal", size: "m", onClick: handleFreshStart }, i18n('fresh-start-button')))))) ||
                        (!disablePromptPresets && (React.createElement("div", { className: cnGptDialog('header-bottom') },
                            React.createElement(PresetList, { disablePromptPresets: disablePromptPresets, promptPresets: promptPresets, onPresetClick: handlePresetClick })))) ||
                        (disablePromptPresets && showedGptAlert && (React.createElement(Alert, { theme: alertTheme, message: alertMessage, align: "center", className: cnGptDialog('description-alert', alertClassName), onClose: onCloseAlert }))))),
            showAnswer && (React.createElement(React.Fragment, null,
                React.createElement("div", { className: cnGptDialog('body') },
                    React.createElement("div", { className: cnGptDialog('answer') }, answerRender(answer))),
                React.createElement("div", { className: cnGptDialog('footer') },
                    showAnswerActions && (React.createElement("div", { className: cnGptDialog('answer-actions') },
                        React.createElement(Button, { className: cnGptDialog('like-button'), view: "normal", size: "m", onClick: handleLike, selected: feedbackType === 'like', loading: feedbackTypeLoading === 'like' },
                            React.createElement(Icon, { data: ThumbsUp, size: 16 })),
                        React.createElement(Button, { className: cnGptDialog('dislike-button'), view: "normal", size: "m", onClick: handleDislike, selected: feedbackType === 'dislike', loading: feedbackTypeLoading === 'dislike' },
                            React.createElement(Icon, { data: ThumbsDown, size: 16 })),
                        feedbackType && (React.createElement("span", { className: cnGptDialog('feedback-message') }, i18n('feedback-message'))))),
                    React.createElement(ActionTooltip, { title: i18n('close-button'), hotkey: gptHotKeys.closeGpt },
                        React.createElement(Button, { className: cnGptDialog('close-button'), view: "flat", size: "m", onClick: onClose }, i18n('close-button'))),
                    React.createElement(ActionTooltip, { title: replaceButtonText, hotkey: gptHotKeys.applyResultGpt },
                        React.createElement(Button, { className: cnGptDialog('apply-button'), view: "action", size: "m", onClick: handleApplyResult }, replaceButtonText)))))));
    }
    return React.createElement("div", { className: cnGptDialog() },
        " ",
        content,
        " ");
};
