import React, { useEffect, useRef, useState } from 'react';
import { getStyles } from '@diplodoc/html-extension';
import { Ellipsis as DotsIcon, Eye } from '@gravity-ui/icons';
import { Button, Icon, Label, Menu, Popup } from '@gravity-ui/uikit';
import debounce from 'lodash/debounce';
import { cn } from '../../../../classname';
import { TextAreaFixed as TextArea } from '../../../../forms/TextInput';
import { i18n } from '../../../../i18n/common';
import { useBooleanState } from '../../../../react-utils/hooks';
import { removeNode } from '../../../../utils/remove-node';
import { YfmHtmlBlockConsts } from '../YfmHtmlBlockSpecs/const';
import './YfmHtmlBlock.css';
export const cnYfmHtmlBlock = cn('yfm-html-block');
export const cnHelper = cn('yfm-html-block-helper');
const b = cnYfmHtmlBlock;
export function generateID() {
    return Math.random().toString(36).substr(2, 8);
}
const DEFAULT_PADDING = 20;
const DEFAULT_DELAY = 100;
const createLinkCLickHandler = (value, document) => (event) => {
    event.preventDefault();
    const targetId = value.getAttribute('href');
    if (targetId) {
        const targetElement = document.querySelector(targetId);
        if (targetElement) {
            targetElement.scrollIntoView({ behavior: 'smooth' });
        }
    }
};
const YfmHtmlBlockPreview = ({ html, onСlick, config }) => {
    var _a, _b, _c, _d, _e, _f;
    const ref = useRef(null);
    const styles = useRef({});
    const classNames = useRef([]);
    const resizeConfig = useRef({});
    const [height, setHeight] = useState('100%');
    useEffect(() => {
        setStyles(config === null || config === void 0 ? void 0 : config.styles);
        setClassNames(config === null || config === void 0 ? void 0 : config.classNames);
    }, [config, (_c = (_b = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contentWindow) === null || _b === void 0 ? void 0 : _b.document) === null || _c === void 0 ? void 0 : _c.body]);
    const handleLoadIFrame = () => {
        var _a;
        const contentWindow = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contentWindow;
        handleResizeIFrame();
        if (contentWindow) {
            const frameDocument = contentWindow.document;
            frameDocument.addEventListener('dblclick', () => {
                onСlick();
            });
        }
    };
    const handleResizeIFrame = () => {
        var _a, _b;
        if (ref.current) {
            const contentWindow = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contentWindow;
            if (contentWindow) {
                const body = contentWindow.document.body;
                if (body) {
                    const height = body.scrollHeight +
                        (((_b = resizeConfig.current) === null || _b === void 0 ? void 0 : _b.padding) || DEFAULT_PADDING) +
                        'px';
                    setHeight(height);
                }
            }
        }
    };
    const setClassNames = (newClassNames) => {
        var _a, _b;
        const body = (_b = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contentWindow) === null || _b === void 0 ? void 0 : _b.document.body;
        if (body && newClassNames) {
            const previousClassNames = classNames.current;
            // remove all classes that were in previousClassNames but are not in classNames
            previousClassNames.forEach((className) => {
                if (!newClassNames.includes(className)) {
                    body.classList.remove(className);
                }
            });
            // add classes that are in classNames
            newClassNames.forEach((className) => {
                if (!body.classList.contains(className)) {
                    body.classList.add(className);
                }
            });
            classNames.current = newClassNames;
        }
    };
    const setStyles = (newStyles) => {
        var _a, _b;
        const body = (_b = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contentWindow) === null || _b === void 0 ? void 0 : _b.document.body;
        if (body && newStyles) {
            const previousStyles = styles.current;
            // remove all styles that are in previousStyles but not in styles
            Object.keys(previousStyles).forEach((property) => {
                if (!Object.prototype.hasOwnProperty.call(newStyles, property)) {
                    body.style.removeProperty(property);
                }
            });
            // sdd or update styles that are in styles
            Object.keys(newStyles).forEach((property) => {
                body.style.setProperty(property, newStyles[property]);
            });
            // update current styles to the new styles
            styles.current = newStyles;
        }
    };
    // finds all relative links (href^="#") and changes their click behavior
    const createAnchorLinkHandlers = (type) => () => {
        var _a;
        const document = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contentWindow.document;
        if (document) {
            document.querySelectorAll('a[href^="#"]').forEach((value) => {
                const handler = createLinkCLickHandler(value, document);
                if (type === 'add') {
                    value.addEventListener('click', handler);
                }
                else {
                    value.removeEventListener('click', handler);
                }
            });
        }
    };
    useEffect(() => {
        var _a, _b;
        (_a = ref.current) === null || _a === void 0 ? void 0 : _a.addEventListener('load', handleLoadIFrame);
        (_b = ref.current) === null || _b === void 0 ? void 0 : _b.addEventListener('load', createAnchorLinkHandlers('add'));
        return () => {
            var _a, _b;
            (_a = ref.current) === null || _a === void 0 ? void 0 : _a.removeEventListener('load', handleLoadIFrame);
            (_b = ref.current) === null || _b === void 0 ? void 0 : _b.removeEventListener('load', createAnchorLinkHandlers('remove'));
        };
    }, [html]);
    useEffect(() => {
        if (ref.current) {
            const resizeObserver = new window.ResizeObserver(debounce(handleResizeIFrame, DEFAULT_DELAY));
            resizeObserver.observe(ref.current);
        }
    }, [(_f = (_e = (_d = ref.current) === null || _d === void 0 ? void 0 : _d.contentWindow) === null || _e === void 0 ? void 0 : _e.document) === null || _f === void 0 ? void 0 : _f.body]);
    return (React.createElement("iframe", { style: {
            height,
        }, ref: ref, title: generateID(), frameBorder: 0, className: b('content'), srcDoc: html }));
};
const CodeEditMode = ({ initialText, onSave, onCancel }) => {
    const [text, setText] = useState(initialText || '\n');
    return (React.createElement("div", { className: b({ editing: true }) },
        React.createElement("div", { className: b('editor') },
            React.createElement(TextArea, { controlProps: {
                    className: cnHelper({ 'prosemirror-stop-event': true }),
                }, value: text, onUpdate: (v) => {
                    setText(v);
                }, autoFocus: true }),
            React.createElement("div", { className: b('controls') },
                React.createElement("div", null,
                    React.createElement(Button, { onClick: onCancel, view: 'flat' },
                        React.createElement("span", { className: cnHelper({ 'prosemirror-stop-event': true }) }, i18n('cancel'))),
                    React.createElement(Button, { onClick: () => onSave(text), view: 'action' },
                        React.createElement("span", { className: cnHelper({ 'prosemirror-stop-event': true }) }, i18n('save'))))))));
};
export const YfmHtmlBlockView = ({ onChange, node, getPos, view, options: { useConfig, sanitize, styles, baseTarget = '_parent', head: headContent = '' }, }) => {
    var _a;
    const [editing, setEditing, unsetEditing, toggleEditing] = useBooleanState(Boolean(node.attrs[YfmHtmlBlockConsts.NodeAttrs.newCreated]));
    const config = useConfig === null || useConfig === void 0 ? void 0 : useConfig();
    const [menuOpen, , , toggleMenuOpen] = useBooleanState(false);
    const buttonRef = useRef(null);
    const handleClick = () => {
        setEditing();
    };
    if (editing) {
        return (React.createElement(CodeEditMode, { initialText: node.attrs[YfmHtmlBlockConsts.NodeAttrs.srcdoc], onCancel: unsetEditing, onSave: (v) => {
                onChange({ [YfmHtmlBlockConsts.NodeAttrs.srcdoc]: v });
                unsetEditing();
            } }));
    }
    let additional = baseTarget ? `<base target="${baseTarget}">` : '';
    if (styles) {
        const stylesContent = typeof styles === 'string'
            ? `<link rel="stylesheet" href="${styles}" />`
            : `<style>${getStyles(styles)}</style>`;
        additional += stylesContent;
    }
    const head = `<head>${headContent || additional}</head>`;
    const body = `<body>${(_a = node.attrs[YfmHtmlBlockConsts.NodeAttrs.srcdoc]) !== null && _a !== void 0 ? _a : ''}</body>`;
    const html = `<!DOCTYPE html><html>${head}${body}</html>`;
    const resultHtml = sanitize ? sanitize(html) : html;
    return (React.createElement("div", { className: b(), onDoubleClick: setEditing },
        React.createElement(Label, { className: b('label'), icon: React.createElement(Icon, { size: 16, data: Eye }) }, i18n('preview')),
        React.createElement(YfmHtmlBlockPreview, { html: resultHtml, "on\u0421lick": handleClick, config: config }),
        React.createElement("div", { className: b('menu') },
            React.createElement(Button, { onClick: toggleMenuOpen, ref: buttonRef, size: 's', className: cnHelper({ 'prosemirror-stop-event': true }) },
                React.createElement(Icon, { data: DotsIcon, className: cnHelper({ 'prosemirror-stop-event': true }) })),
            React.createElement(Popup, { anchorRef: buttonRef, open: menuOpen, onClose: toggleMenuOpen, placement: ['bottom-end'] },
                React.createElement(Menu, null,
                    React.createElement(Menu.Item, { onClick: () => {
                            toggleEditing();
                            toggleMenuOpen();
                        } }, i18n('edit')),
                    React.createElement(Menu.Item, { onClick: () => {
                            const pos = getPos();
                            if (pos === undefined)
                                return;
                            removeNode({
                                node,
                                pos,
                                tr: view.state.tr,
                                dispatch: view.dispatch,
                            });
                        } }, i18n('remove')))))));
};
