import { __rest } from "tslib";
import React, { useRef, useState } from 'react';
import { ChevronDown, ChevronUp, Xmark } from '@gravity-ui/icons';
import { Button, Card, Checkbox, Icon, Popup, TextInput, sp, } from '@gravity-ui/uikit';
import { cn } from '../../../../classname';
import { i18n } from '../../../../i18n/search';
import { enterKeyHandler } from '../../../../utils/handlers';
import './SearchPopup.css';
const b = cn('search-card');
const noop = () => { };
export const SearchCard = ({ onChange = noop, onClose = noop, onSearchPrev = noop, onSearchNext = noop, onConfigChange = noop, }) => {
    const [query, setQuery] = useState('');
    const [isCaseSensitive, setIsCaseSensitive] = useState(false);
    const [isWholeWord, setIsWholeWord] = useState(false);
    const textInputRef = useRef(null);
    const setInputFocus = () => {
        var _a;
        (_a = textInputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    };
    const handleInputChange = (event) => {
        const { target: { value }, } = event;
        setQuery(value);
        onChange(value);
    };
    const handleClose = () => {
        setQuery('');
        onClose(query);
        setInputFocus();
    };
    const handlePrev = () => {
        onSearchPrev(query);
        setInputFocus();
    };
    const handleNext = () => {
        onSearchNext(query);
        setInputFocus();
    };
    const handleIsCaseSensitive = () => {
        onConfigChange({
            isCaseSensitive: !isCaseSensitive,
            isWholeWord,
        });
        setIsCaseSensitive((isCaseSensitive) => !isCaseSensitive);
        setInputFocus();
    };
    const handleIsWholeWord = () => {
        onConfigChange({
            isCaseSensitive,
            isWholeWord: !isWholeWord,
        });
        setIsWholeWord((isWholeWord) => !isWholeWord);
        setInputFocus();
    };
    const handleSearchKeyPress = enterKeyHandler(handleNext);
    return (React.createElement(Card, { className: b() },
        React.createElement("div", { className: b('header') },
            React.createElement("span", { className: b('title') },
                " ",
                i18n('title')),
            React.createElement(Button, { onClick: handleClose, size: "s", view: "flat" },
                React.createElement(Icon, { data: Xmark, size: 14 }))),
        React.createElement(TextInput, { controlRef: textInputRef, className: sp({ mb: 2 }), size: "s", autoFocus: true, onKeyPress: handleSearchKeyPress, onChange: handleInputChange, value: query, endContent: React.createElement(React.Fragment, null,
                React.createElement(Button, { onClick: handlePrev },
                    React.createElement(Icon, { data: ChevronUp, size: 12 })),
                React.createElement(Button, { onClick: handleNext },
                    React.createElement(Icon, { data: ChevronDown, size: 12 }))) }),
        React.createElement(Checkbox, { size: "m", onUpdate: handleIsCaseSensitive, checked: isCaseSensitive, className: sp({ mr: 4 }) }, i18n('label_case-sensitive')),
        React.createElement(Checkbox, { size: "m", onUpdate: handleIsWholeWord, checked: isWholeWord }, i18n('label_whole-word'))));
};
export const SearchPopup = (_a) => {
    var { open, anchor, onClose } = _a, props = __rest(_a, ["open", "anchor", "onClose"]);
    const anchorRef = useRef(anchor);
    return (React.createElement(Popup, { onEscapeKeyDown: onClose, open: anchorRef.current && open, anchorRef: anchorRef, placement: "bottom-end" },
        React.createElement(SearchCard, Object.assign({ onClose: onClose }, props))));
};
SearchPopup.displayName = 'SearchPopup';
export function renderSearchPopup(_a) {
    var { anchor, open, onClose } = _a, props = __rest(_a, ["anchor", "open", "onClose"]);
    return (React.createElement(React.Fragment, null, anchor && React.createElement(SearchPopup, Object.assign({ open: open, onClose: onClose, anchor: anchor }, props))));
}
